@@ -6,7 +6,7 @@ require 'jirametrics/status_collection'
 class ProjectConfig
   attr_reader :target_path, :jira_config, :all_boards, :possible_statuses,
     :download_config, :file_configs, :exporter, :data_version, :name, :board_configs,
-    :settings, :aggregate_config, :discarded_changes_data
+    :settings, :aggregate_config, :discarded_changes_data, :users
   attr_accessor :time_range, :jira_url, :id
 
   def initialize exporter:, jira_config:, block:, target_path: '.', name: '', id: nil
@@ -40,6 +40,7 @@ class ProjectConfig
     @id = guess_project_id
     load_project_metadata
     load_sprints
+    load_users
   end
 
   def run load_only: false
@@ -113,10 +114,14 @@ class ProjectConfig
   def file_prefix prefix
     # The file_prefix has to be set before almost everything else. It really should have been an attribute
     # on the project declaration itself. Hindsight is 20/20.
+
+    # There can only be one of these
     if @file_prefix
-      raise "file_prefix should only be set once. Was #{@file_prefix.inspect} and now changed to #{prefix.inspect}."
+      raise "file_prefix can only be set once. Was #{@file_prefix.inspect} and now changed to #{prefix.inspect}."
     end
 
+    raise_if_prefix_already_used(prefix)
+
     @file_prefix = prefix
 
     # Yes, this is a wierd place to be initializing this. Unfortunately, it has to happen after the file_prefix
@@ -129,8 +134,21 @@ class ProjectConfig
     @file_prefix
   end
 
-  def get_file_prefix # rubocop:disable Naming/AccessorMethodName
-    raise 'file_prefix has not been set yet. Move it to the top of the project declaration.' unless @file_prefix
+  def raise_if_prefix_already_used prefix
+    @exporter.project_configs.each do |project|
+      next unless project.get_file_prefix(raise_if_not_set: false) == prefix && project.target_path == target_path
+
+      raise "Project #{name.inspect} specifies file prefix #{prefix.inspect}, " \
+        "but that is already used by project #{project.name.inspect} in the same target path #{target_path.inspect}. " \
+        'This is almost guaranteed to be too much copy and paste in your configuration. ' \
+        'File prefixes must be unique within a directory.'
+    end
+  end
+
+  def get_file_prefix raise_if_not_set: true
+    if @file_prefix.nil? && raise_if_not_set
+      raise 'file_prefix has not been set yet. Move it to the top of the project declaration.'
+    end
 
     @file_prefix
   end
@@ -151,6 +169,8 @@ class ProjectConfig
   end
 
   def status_category_mapping status:, category:
+    return if @exporter.downloading?
+
     status, status_id = possible_statuses.parse_name_id status
     category, category_id = possible_statuses.parse_name_id category
 
@@ -323,6 +343,21 @@ class ProjectConfig
     raise
   end
 
+  def load_users
+    @users = []
+    filename = File.join @target_path, "#{get_file_prefix}_users.json"
+    return unless File.exist? filename
+
+    json = file_system.load_json(filename)
+    json.each { |user_data| @users << User.new(raw: user_data) }
+  end
+
+  def atlassian_document_format
+    @atlassian_document_format ||= AtlassianDocumentFormat.new(
+      users: @users, timezone_offset: exporter.timezone_offset
+    )
+  end
+
   def to_time string, end_of_day: false
     time = end_of_day ? '23:59:59' : '00:00:00'
     string = "#{string}T#{time}#{exporter.timezone_offset}" if string.match?(/^\d{4}-\d{2}-\d{2}$/)
@@ -356,7 +391,7 @@ class ProjectConfig
 
   # To be used by the aggregate_config only. Not intended to be part of the public API
   def add_issues issues_list
-    @issues = [] if @issues.nil?
+    @issues = IssueCollection.new if @issues.nil?
     @all_boards = {}
 
     issues_list.each do |issue|
@@ -373,7 +408,7 @@ class ProjectConfig
           'declaration but none are here. Check your config.'
       end
 
-      return @issues = [] if @exporter.downloading?
+      return @issues = IssueCollection.new if @exporter.downloading?
       raise 'No data found. Must do a download before an export' unless data_downloaded?
 
       load_data if all_boards.empty?
@@ -385,7 +420,7 @@ class ProjectConfig
         issues = load_issues_from_issues_directory path: issues_path, timezone_offset: timezone_offset
       else
         file_system.log "Can't find directory #{issues_path}. Has a download been done?", also_write_to_stderr: true
-        return []
+        return IssueCollection.new
       end
 
       # Attach related issues
@@ -397,7 +432,8 @@ class ProjectConfig
 
       # We'll have some issues that are in the list that weren't part of the initial query. Once we've
       # attached them in the appropriate places, remove any that aren't part of that initial set.
-      @issues = issues.select { |i| i.in_initial_query? }
+      issues.reject! { |i| !i.in_initial_query? } # rubocop:disable Style/InverseMethods
+      @issues = issues
     end
 
     @issues
@@ -438,7 +474,7 @@ class ProjectConfig
   end
 
   def load_issues_from_issues_directory path:, timezone_offset:
-    issues = []
+    issues = IssueCollection.new
     default_board = nil
 
     group_filenames_and_board_ids(path: path).each do |filename, board_ids|
@@ -450,6 +486,10 @@ class ProjectConfig
       end
 
       boards.each do |board|
+        if board.cycletime.nil?
+          raise "The board declaration for board #{board.id} must come before the " \
+            "first usage of 'issues' in the configuration"
+        end
         issues << Issue.new(raw: content, timezone_offset: timezone_offset, board: board)
       end
     end
@@ -462,7 +502,7 @@ class ProjectConfig
   # board ids appropriately.
   def group_filenames_and_board_ids path:
     hash = {}
-    Dir.foreach(path) do |filename|
+    file_system.foreach(path) do |filename|
       # Matches either FAKE-123.json or FAKE-123-456.json
       if /^(?<key>[^-]+-\d+)(?<_>-(?<board_id>\d+))?\.json$/ =~ filename
         (hash[key] ||= []) << [filename, board_id&.to_i || :unknown]
